@@ -7,7 +7,10 @@ _thumb: .4, .4
 import seaborn as sns
 sns.set(style="ticks")
 
+# Load the example dataset for Anscombe's quartet
 df = sns.load_dataset("anscombe")
-sns.lmplot("x", "y", col="dataset", hue="dataset", data=df,
+
+# Show the results of a linear regression within each dataset
+sns.lmplot(x="x", y="y", col="dataset", hue="dataset", data=df,
            col_wrap=2, ci=None, palette="muted", size=4,
            scatter_kws={"s": 50, "alpha": 1})
