@@ -6,11 +6,15 @@ Paired discrete plots
 import seaborn as sns
 sns.set(style="whitegrid")
 
+# Load the example Titanic dataset
 titanic = sns.load_dataset("titanic")
 
+# Set up a grid to plot survival probability against several variables
 g = sns.PairGrid(titanic, y_vars="survived",
                  x_vars=["class", "sex", "who", "alone"],
                  size=5, aspect=.5)
+
+# Draw a seaborn pointplot onto each Axes
 g.map(sns.pointplot, color=sns.xkcd_rgb["plum"])
 g.set(ylim=(0, 1))
-sns.despine(left=True)
+sns.despine(fig=g.fig, left=True)
