@@ -2,11 +2,11 @@
 Linear regression with marginal distributions
 =============================================
 
+_thumb: .5, .6
 """
 import seaborn as sns
-sns.set(style="darkgrid")
+sns.set(style="darkgrid", color_codes=True)
 
 tips = sns.load_dataset("tips")
-color = sns.color_palette()[2]
 g = sns.jointplot("total_bill", "tip", data=tips, kind="reg",
-                  xlim=(0, 60), ylim=(0, 12), color=color, size=7)
+                  xlim=(0, 60), ylim=(0, 12), color="r", size=7)
