@@ -2,12 +2,14 @@ from .rcmod import *
 from .utils import *
 from .palettes import *
 from .linearmodels import *
+from .categorical import *
 from .distributions import *
 from .timeseries import *
 from .matrix import *
 from .miscplot import *
 from .axisgrid import *
 from .xkcd_rgb import xkcd_rgb
+from .crayons import crayons
 set()
 
-__version__ = "0.5.1"
+__version__ = "0.6.0"
