@@ -200,6 +200,22 @@ class TestHeatmap(object):
         nt.assert_equal(p.xticklabels, xticklabels)
         nt.assert_equal(p.yticklabels, yticklabels[::-1])
 
+    def test_custom_ticklabel_interval(self):
+
+        kws = self.default_kws.copy()
+        kws['xticklabels'] = 2
+        kws['yticklabels'] = 3
+        p = mat._HeatMapper(self.df_norm, **kws)
+
+        nx, ny = self.df_norm.T.shape
+        ystart = (ny - 1) % 3
+        npt.assert_array_equal(p.xticks, np.arange(0, nx, 2) + .5)
+        npt.assert_array_equal(p.yticks, np.arange(ystart, ny, 3) + .5)
+        npt.assert_array_equal(p.xticklabels,
+                               self.df_norm.columns[::2])
+        npt.assert_array_equal(p.yticklabels,
+                               self.df_norm.index[::-1][ystart:ny:3])
+
     def test_heatmap_annotation(self):
 
         ax = mat.heatmap(self.df_norm, annot=True, fmt=".1f",
@@ -207,6 +223,20 @@ class TestHeatmap(object):
         for val, text in zip(self.x_norm[::-1].flat, ax.texts):
             nt.assert_equal(text.get_text(), "{:.1f}".format(val))
             nt.assert_equal(text.get_fontsize(), 14)
+        plt.close("all")
+
+    def test_heatmap_annotation_with_mask(self):
+
+        df = pd.DataFrame(data={'a': [1, 1, 1],
+                                'b': [2, np.nan, 2],
+                                'c': [3, 3, np.nan]})
+        mask = np.isnan(df.values)
+        df_masked = np.ma.masked_where(mask, df)
+        ax = mat.heatmap(df, annot=True, fmt='.1f', mask=mask)
+        nt.assert_equal(len(df_masked[::-1].compressed()), len(ax.texts))
+        for val, text in zip(df_masked[::-1].compressed(), ax.texts):
+            nt.assert_equal("{:.1f}".format(val), text.get_text())
+        plt.close("all")
 
     def test_heatmap_cbar(self):
 
@@ -286,6 +316,31 @@ class TestHeatmap(object):
         nt.assert_equal(ax.get_aspect(), "equal")
         plt.close("all")
 
+    def test_mask_validation(self):
+
+        mask = mat._matrix_mask(self.df_norm, None)
+        nt.assert_equal(mask.shape, self.df_norm.shape)
+        nt.assert_equal(mask.values.sum(), 0)
+
+        with nt.assert_raises(ValueError):
+            bad_array_mask = self.rs.randn(3, 6) > 0
+            mat._matrix_mask(self.df_norm, bad_array_mask)
+
+        with nt.assert_raises(ValueError):
+            bad_df_mask = pd.DataFrame(self.rs.randn(4, 8) > 0)
+            mat._matrix_mask(self.df_norm, bad_df_mask)
+
+    def test_missing_data_mask(self):
+
+        data = pd.DataFrame(np.arange(4, dtype=np.float).reshape(2, 2))
+        data.loc[0, 0] = np.nan
+        mask = mat._matrix_mask(data, None)
+        npt.assert_array_equal(mask, [[True, False], [False, False]])
+
+        mask_in = np.array([[False, True], [False, False]])
+        mask_out = mat._matrix_mask(data, mask_in)
+        npt.assert_array_equal(mask_out, [[True, True], [False, False]])
+
 
 class TestDendrogram(object):
     rs = np.random.RandomState(sum(map(ord, "dendrogram")))
@@ -559,7 +614,7 @@ class TestClustermap(object):
     default_plot_kws = dict(metric='euclidean', method='average',
                             colorbar_kws=None,
                             row_cluster=True, col_cluster=True,
-                            row_linkage=None, col_linkage=None, mask=None)
+                            row_linkage=None, col_linkage=None)
 
     row_colors = color_palette('Set2', df_norm.shape[0])
     col_colors = color_palette('Dark2', df_norm.shape[1])
@@ -801,3 +856,63 @@ class TestClustermap(object):
         nt.assert_equal(len(cm.ax_col_colors.collections), 1)
 
         plt.close('all')
+
+    def test_cluster_false_row_col_colors(self):
+        kws = self.default_kws.copy()
+        kws['row_cluster'] = False
+        kws['col_cluster'] = False
+        kws['row_colors'] = self.row_colors
+        kws['col_colors'] = self.col_colors
+
+        cm = mat.clustermap(self.df_norm, **kws)
+        nt.assert_equal(len(cm.ax_row_dendrogram.lines), 0)
+        nt.assert_equal(len(cm.ax_col_dendrogram.lines), 0)
+
+        nt.assert_equal(len(cm.ax_row_dendrogram.get_xticks()), 0)
+        nt.assert_equal(len(cm.ax_row_dendrogram.get_yticks()), 0)
+        nt.assert_equal(len(cm.ax_col_dendrogram.get_xticks()), 0)
+        nt.assert_equal(len(cm.ax_col_dendrogram.get_yticks()), 0)
+        nt.assert_equal(len(cm.ax_row_colors.collections), 1)
+        nt.assert_equal(len(cm.ax_col_colors.collections), 1)
+
+        pdt.assert_frame_equal(cm.data2d, self.df_norm)
+        plt.close('all')
+
+    def test_mask_reorganization(self):
+
+        kws = self.default_kws.copy()
+        kws["mask"] = self.df_norm > 0
+
+        g = mat.clustermap(self.df_norm, **kws)
+        npt.assert_array_equal(g.data2d.index, g.mask.index)
+        npt.assert_array_equal(g.data2d.columns, g.mask.columns)
+
+        npt.assert_array_equal(g.mask.index,
+                               self.df_norm.index[
+                                   g.dendrogram_row.reordered_ind])
+        npt.assert_array_equal(g.mask.columns,
+                               self.df_norm.columns[
+                                   g.dendrogram_col.reordered_ind])
+
+        plt.close("all")
+
+    def test_ticklabel_reorganization(self):
+
+        kws = self.default_kws.copy()
+        xtl = np.arange(self.df_norm.shape[1])
+        kws["xticklabels"] = list(xtl)
+        ytl = self.letters.ix[:self.df_norm.shape[0]]
+        kws["yticklabels"] = ytl
+
+        g = mat.clustermap(self.df_norm, **kws)
+
+        xtl_actual = [t.get_text() for t in g.ax_heatmap.get_xticklabels()]
+        ytl_actual = [t.get_text() for t in g.ax_heatmap.get_yticklabels()]
+
+        xtl_want = xtl[g.dendrogram_col.reordered_ind].astype("<U1")
+        ytl_want = ytl[g.dendrogram_row.reordered_ind].astype("<U1")[::-1]
+
+        npt.assert_array_equal(xtl_actual, xtl_want)
+        npt.assert_array_equal(ytl_actual, ytl_want)
+
+        plt.close("all")
