@@ -1,12 +1,27 @@
 """Tests for plotting utilities."""
 import warnings
+import tempfile
+import shutil
 
 import numpy as np
+import pandas as pd
 import matplotlib.pyplot as plt
-from numpy.testing import assert_array_equal
 import nose
 import nose.tools as nt
 from nose.tools import assert_equal, raises
+import numpy.testing as npt
+import pandas.util.testing as pdt
+
+from distutils.version import LooseVersion
+pandas_has_categoricals = LooseVersion(pd.__version__) >= "0.15"
+
+from pandas.util.testing import network
+from ..utils import get_dataset_names, load_dataset
+
+try:
+    from bs4 import BeautifulSoup
+except ImportError:
+    BeautifulSoup = None
 
 from .. import utils, rcmod
 
@@ -57,7 +72,7 @@ def test_ci_to_errsize():
                                [.25, 0]])
 
     test_errsize = utils.ci_to_errsize(cis, heights)
-    assert_array_equal(actual_errsize, test_errsize)
+    npt.assert_array_equal(actual_errsize, test_errsize)
 
 
 def test_desaturate():
@@ -183,6 +198,28 @@ class TestSpineUtils(object):
 
         plt.close("all")
 
+    def test_despine_trim_inverted(self):
+
+        f, ax = plt.subplots()
+        ax.plot([1, 2, 3], [1, 2, 3])
+        ax.set_ylim(.85, 3.15)
+        ax.invert_yaxis()
+
+        utils.despine(trim=True)
+        for side in self.inner_sides:
+            bounds = ax.spines[side].get_bounds()
+            nt.assert_equal(bounds, (1, 3))
+
+        plt.close("all")
+
+    def test_despine_trim_noticks(self):
+
+        f, ax = plt.subplots()
+        ax.plot([1, 2, 3], [1, 2, 3])
+        ax.set_yticks([])
+        utils.despine(trim=True)
+        nt.assert_equal(ax.get_yticks().size, 0)
+
     def test_offset_spines_warns(self):
         with warnings.catch_warnings(record=True) as w:
             warnings.simplefilter("always", category=UserWarning)
@@ -244,3 +281,95 @@ def test_ticklabels_overlap():
     x, y = utils.axes_ticklabels_overlap(ax)
     assert x
     assert not y
+
+
+def test_categorical_order():
+
+    x = ["a", "c", "c", "b", "a", "d"]
+    order = ["a", "b", "c", "d"]
+
+    out = utils.categorical_order(x)
+    nt.assert_equal(out, ["a", "c", "b", "d"])
+
+    out = utils.categorical_order(x, order)
+    nt.assert_equal(out, order)
+
+    out = utils.categorical_order(x, ["b", "a"])
+    nt.assert_equal(out, ["b", "a"])
+
+    out = utils.categorical_order(np.array(x))
+    nt.assert_equal(out, ["a", "c", "b", "d"])
+
+    out = utils.categorical_order(pd.Series(x))
+    nt.assert_equal(out, ["a", "c", "b", "d"])
+
+    if pandas_has_categoricals:
+        x = pd.Categorical(x, order)
+        out = utils.categorical_order(x)
+        nt.assert_equal(out, list(x.categories))
+
+        x = pd.Series(x)
+        out = utils.categorical_order(x)
+        nt.assert_equal(out, list(x.cat.categories))
+
+        out = utils.categorical_order(x, ["b", "a"])
+        nt.assert_equal(out, ["b", "a"])
+
+    x = ["a", np.nan, "c", "c", "b", "a", "d"]
+    out = utils.categorical_order(x)
+    nt.assert_equal(out, ["a", "c", "b", "d"])
+
+
+if LooseVersion(pd.__version__) >= "0.15":
+
+    def check_load_dataset(name):
+        ds = load_dataset(name, cache=False)
+        assert(isinstance(ds, pd.DataFrame))
+
+    def check_load_cached_dataset(name):
+        # Test the cacheing using a temporary file.
+        # With Python 3.2+, we could use the tempfile.TemporaryDirectory()
+        # context manager instead of this try...finally statement
+        tmpdir = tempfile.mkdtemp()
+        try:
+            # download and cache
+            ds = load_dataset(name, cache=True, data_home=tmpdir)
+
+            # use cached version
+            ds2 = load_dataset(name, cache=True, data_home=tmpdir)
+            pdt.assert_frame_equal(ds, ds2)
+
+        finally:
+            shutil.rmtree(tmpdir)
+
+    @network(url="https://github.com/mwaskom/seaborn-data")
+    def test_get_dataset_names():
+        if not BeautifulSoup:
+            raise nose.SkipTest("No BeautifulSoup available for parsing html")
+        names = get_dataset_names()
+        assert(len(names) > 0)
+        assert(u"titanic" in names)
+
+    @network(url="https://github.com/mwaskom/seaborn-data")
+    def test_load_datasets():
+        if not BeautifulSoup:
+            raise nose.SkipTest("No BeautifulSoup available for parsing html")
+
+        # Heavy test to verify that we can load all available datasets
+        for name in get_dataset_names():
+            # unfortunately @network somehow obscures this generator so it
+            # does not get in effect, so we need to call explicitly
+            # yield check_load_dataset, name
+            check_load_dataset(name)
+
+    @network(url="https://github.com/mwaskom/seaborn-data")
+    def test_load_cached_datasets():
+        if not BeautifulSoup:
+            raise nose.SkipTest("No BeautifulSoup available for parsing html")
+
+        # Heavy test to verify that we can load all available datasets
+        for name in get_dataset_names():
+            # unfortunately @network somehow obscures this generator so it
+            # does not get in effect, so we need to call explicitly
+            # yield check_load_dataset, name
+            check_load_cached_dataset(name)
